<?php

namespace Survey\Benchmark\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Survey\Benchmark\Http\Requests\BenchmarkRequest;
use Survey\Benchmark\Interfaces\BenchmarkRepositoryInterface;
use Survey\Benchmark\Models\Benchmark;

/**
 * Resource controller class for benchmark.
 */
class BenchmarkResourceController extends BaseController
{

    /**
     * Initialize benchmark resource controller.
     *
     * @param type BenchmarkRepositoryInterface $benchmark
     *
     * @return null
     */
    public function __construct(BenchmarkRepositoryInterface $benchmark)
    {
        parent::__construct();
        $this->repository = $benchmark;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Benchmark\Repositories\Criteria\BenchmarkResourceCriteria::class);
    }

    /**
     * Display a list of benchmark.
     *
     * @return Response
     */
    public function index(BenchmarkRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Survey\Benchmark\Repositories\Presenter\BenchmarkPresenter::class)
                ->$function();
        }

        $benchmarks = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('benchmark::benchmark.names'))
            ->view('benchmark::benchmark.index', true)
            ->data(compact('benchmarks', 'view'))
            ->output();
    }

    /**
     * Display benchmark.
     *
     * @param Request $request
     * @param Model   $benchmark
     *
     * @return Response
     */
    public function show(BenchmarkRequest $request, Benchmark $benchmark)
    {

        if ($benchmark->exists) {
            $view = 'benchmark::benchmark.show';
        } else {
            $view = 'benchmark::benchmark.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('benchmark::benchmark.name'))
            ->data(compact('benchmark'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new benchmark.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BenchmarkRequest $request)
    {

        $benchmark = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('benchmark::benchmark.name')) 
            ->view('benchmark::benchmark.create', true) 
            ->data(compact('benchmark'))
            ->output();
    }

    /**
     * Create new benchmark.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BenchmarkRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $benchmark                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('benchmark::benchmark.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('benchmark/benchmark/' . $benchmark->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/benchmark/benchmark'))
                ->redirect();
        }

    }

    /**
     * Show benchmark for editing.
     *
     * @param Request $request
     * @param Model   $benchmark
     *
     * @return Response
     */
    public function edit(BenchmarkRequest $request, Benchmark $benchmark)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('benchmark::benchmark.name'))
            ->view('benchmark::benchmark.edit', true)
            ->data(compact('benchmark'))
            ->output();
    }

    /**
     * Update the benchmark.
     *
     * @param Request $request
     * @param Model   $benchmark
     *
     * @return Response
     */
    public function update(BenchmarkRequest $request, Benchmark $benchmark)
    {
        try {
            $attributes = $request->all();

            $benchmark->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('benchmark::benchmark.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('benchmark/benchmark/' . $benchmark->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('benchmark/benchmark/' . $benchmark->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the benchmark.
     *
     * @param Model   $benchmark
     *
     * @return Response
     */
    public function destroy(BenchmarkRequest $request, Benchmark $benchmark)
    {
        try {

            $benchmark->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('benchmark::benchmark.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('benchmark/benchmark/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('benchmark/benchmark/' . $benchmark->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple benchmark.
     *
     * @param Model   $benchmark
     *
     * @return Response
     */
    public function delete(BenchmarkRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('benchmark::benchmark.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('benchmark/benchmark'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/benchmark/benchmark'))
                ->redirect();
        }

    }

    /**
     * Restore deleted benchmarks.
     *
     * @param Model   $benchmark
     *
     * @return Response
     */
    public function restore(BenchmarkRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('benchmark::benchmark.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/benchmark/benchmark'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/benchmark/benchmark/'))
                ->redirect();
        }

    }

}
